# frozen_string_literal: true

require 'apiphobic/tokens/invalid'
require 'apiphobic/tokens/base64'
require 'apiphobic/tokens/base64s/null'
require 'apiphobic/tokens/json_web_token'
require 'apiphobic/tokens/json_web_tokens/null'

module  Apiphobic
module  Requests
class   AuthorizationToken
  BASE64_PATTERN                = %r{[A-Za-z0-9_/\+\=\-\.]}
  BASE64_TOKEN_HEADER_PATTERN   = /\A(?:Basic|Bearer)\s+(.*)\z/
  BASE64_TOKEN_PARAM_NAME       = 'token_b64'
  BASE64_TOKEN_PARAM_PATTERN    = /(?:\A|&)#{BASE64_TOKEN_PARAM_NAME}=(.*)(?=\z|&)/

  JSON_WEB_TOKEN_PATTERN        = /(#{BASE64_PATTERN}+?\.){4}#{BASE64_PATTERN}+?/
  JSON_WEB_TOKEN_HEADER_PATTERN = /\AToken\s+(.*)\z/
  JSON_WEB_TOKEN_PARAM_NAME     = 'token_jwt'
  JSON_WEB_TOKEN_PARAM_PATTERN  = /(?:\A|&)#{JSON_WEB_TOKEN_PARAM_NAME}=(.*)(?=\z|&)/

  attr_accessor :request,
                :private_key,
                :type

  def initialize(request,
                 private_key:,
                 type:)

    self.request     = request
    self.private_key = private_key
    self.type        = type
  end

  def fetch
    if !token_from_header.blank? && token_from_params.blank?
      token_from_header
    elsif !token_from_params.blank?
      token_from_params
    else
      token_from_header
    end
  end

  private

  def token_from_header
    @token_from_header ||= if header_contains_json_web_token?
                             retrieve_token(json_web_token_from_header)
                           elsif header_contains_base64_token?
                             Tokens::Base64.convert(raw_token: base64_token_from_header)
                           else
                             Tokens::Null.instance
                           end
  rescue Erratum::Errors::InvalidToken
    Tokens::Invalid.instance
  end

  def token_from_params
    @token_from_params ||= if params_contains_json_web_token?
                             retrieve_token(json_web_token_from_params)
                           elsif params_contains_base64_token?
                             Tokens::Base64.convert(raw_token: base64_token_from_params)
                           else
                             Tokens::Null.instance
                           end
  rescue Erratum::Errors::InvalidToken
    Tokens::Invalid.instance
  end

  def base64_token_from_header
    raw_authorization_header[BASE64_TOKEN_HEADER_PATTERN, 1]
  end

  def base64_token_from_params
    if request.respond_to?(:params)
      request.params[BASE64_TOKEN_PARAM_NAME] || ''
    else
      request['QUERY_STRING'][BASE64_TOKEN_PARAM_PATTERN, 1] || ''
    end
  end

  def json_web_token_from_header
    raw_authorization_header[JSON_WEB_TOKEN_HEADER_PATTERN, 1]
  end

  def json_web_token_from_params
    if request.respond_to?(:params)
      request.params[JSON_WEB_TOKEN_PARAM_NAME] || ''
    else
      request['QUERY_STRING'][JSON_WEB_TOKEN_PARAM_PATTERN, 1] || ''
    end
  end

  def header_contains_base64_token?
    raw_authorization_header =~ BASE64_TOKEN_HEADER_PATTERN
  end

  def header_contains_json_web_token?
    raw_authorization_header =~ JSON_WEB_TOKEN_HEADER_PATTERN
  end

  def params_contains_base64_token?
    if request.respond_to?(:params)
      request.params.has_key?(BASE64_TOKEN_PARAM_NAME)
    else
      request['QUERY_STRING'] =~ BASE64_TOKEN_PARAM_PATTERN
    end
  end

  def params_contains_json_web_token?
    if request.respond_to?(:params)
      request.params.has_key?(JSON_WEB_TOKEN_PARAM_NAME)
    else
      request['QUERY_STRING'] =~ JSON_WEB_TOKEN_PARAM_PATTERN
    end
  end

  def raw_authorization_header
    if request.respond_to?(:headers)
      request.headers['HTTP_AUTHORIZATION'] || ''
    else
      request['HTTP_AUTHORIZATION'] || ''
    end
  end

  def retrieve_token(token)
    Tokens::JsonWebToken.__send__(
      "from_#{type.downcase}",
      token,
      private_key: private_key,
    )
  end
end
end
end
